// Copyright 2024 XMOS LIMITED.
// This Software is subject to the terms of the XMOS Public Licence: Version 1.

#ifndef _APP_CONFIG_H_
#define _APP_CONFIG_H_

#include "app_build_config.h"

#include "xk_audio_316_mc_ab/board.h"

#define ADAT_MAX_SAMPLES                                    8 // Max channels
#define ADAT_MULTIPLIER                                     512 // For 22/24M master clock

#define SPDIF_MAX_SAMPLES                                   2 // Max channels

#define I2S_DATA_BITS                                       32

#define FORMAT_CHANGE_MUTE_MS                               250 // How long to mute after format change
#define I2S_RATE_MEASUREMENT_HZ                             100 // How often to count samples to estimate sample rate

#define ADAT_RATE_MEASUREMENT_HZ                            100 // How often to count samples to estimate sample rate

#define MAX_FIFO_LENGTH                                     80 // Sized to allow 192 kHz and 350 PPM. Please see lib_src ASRC task docs for optimisation of this.

#if (CONFIG == ADAT)

#define NUM_I2S_DAC_LINES                                   4
#define NUM_I2S_ADC_LINES                                   4

#include "adat_wrapper.h"
#define c_smux_change_adat_rx_name                          c_smux_change_adat_rx
#define c_smux_change_adat_rx_declaration                   chan c_smux_change_adat_rx
#define chan_c_dig_tx_name                                  c_dig_tx
#define chan_c_dig_tx_declaration                           chan c_dig_tx

#elif (CONFIG == SPDIF)

#define NUM_I2S_DAC_LINES                                   1
#define NUM_I2S_ADC_LINES                                   1

#include "spdif_wrapper.h"
#define c_smux_change_adat_rx_name                          NULL
#define c_smux_change_adat_rx_declaration
#define chan_c_dig_tx_name                                  c_dig_tx
#define chan_c_dig_tx_declaration                           chan c_dig_tx

#elif (CONFIG == I2S)

#define NUM_I2S_IN_DAC_LINES                                0
#define NUM_I2S_IN_ADC_LINES                                1
#define NUM_I2S_DAC_LINES                                   1
#define NUM_I2S_ADC_LINES                                   0

#include "i2s_in_wrapper.h"
#define c_smux_change_adat_rx_name                          NULL
#define c_smux_change_adat_rx_declaration
#define chan_c_dig_tx_name                                  NULL
#define chan_c_dig_tx_declaration
#else
#error CONFIG not supported
#endif

// For I2S master only (currently the CODEC is setup to be this). Can be removed when deployed with external I2S master
#define MCLK_441                                            22579200
#define MCLK_48                                             24576000
#define DEFAULT_FREQ                                        44100


// Board configuration from lib_board_support
static const xk_audio_316_mc_ab_config_t hw_config = {
        CLK_FIXED,          // clk_mode
        1,                  // dac_is_clk_master
        (DEFAULT_FREQ % 22050 == 0) ? MCLK_441 : MCLK_48,         // default_mclk
        0,                  // pll_sync_freq (unused)
        0,                  // pcm_format, 0 = I2S
        32,                 // i2s_n_bits
        2                   // i2s_chans_per_frame
};

#endif
